# coding: utf-8
from sqlalchemy import BigInteger, Column, Date, DateTime, Float, ForeignKey, Index, Integer, SmallInteger, String, Table, Text, text
from sqlalchemy.dialects.mysql.types import BIT, MEDIUMBLOB
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()
metadata = Base.metadata


class AbstractBug(Base):
    __tablename__ = 'AbstractBug'

    bug_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    id = Column(Integer, nullable=False)
    status = Column(String(255), nullable=False)
    statusNumber = Column(Integer, nullable=False, index=True)
    type = Column(String(255), nullable=False)
    typeNumber = Column(Integer, nullable=False, index=True)
    issueDate = Column(Date, index=True)
    completeDate = Column(Date, index=True)
    author = Column(String(255))
    location = Column(String(255))
    description = Column(String)
    resolution = Column(String)
    versionNumber = Column(String(255))
    osName = Column(String(255))
    email = Column(String)


class GBTVEGASSpectrometer(Base):
    __tablename__ = 'GBT_VEGAS_Spectrometer'

    id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    mode = Column(Integer, nullable=False)
    bandwidth = Column(Float(asdecimal=True), nullable=False)
    restFrequencies = Column(String(255), nullable=False)
    spectralResolution = Column(Float(asdecimal=True), nullable=False)
    integrationTime = Column(Float(asdecimal=True), nullable=False)
    dataRate = Column(Float(asdecimal=True), nullable=False)
    vegas_id = Column(ForeignKey('GBT_VEGAS_14A.Id', ondelete='CASCADE'), index=True)

    vegas = relationship('GBTVEGAS14A')


class GBTVegasPulsar(Base):
    __tablename__ = 'GBT_Vegas_Pulsar'

    Id = Column(BigInteger, primary_key=True)
    COMMENTS = Column(String(512))
    BAND_WIDTH = Column(String(128))
    BAND_WIDTH_UNIT = Column(String(255))
    OBSERVING_MODE = Column(String(128))
    POLAR_CROSS_PROD = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    Dedispersion = Column(String(255))
    BITS = Column(String(255))


class NormalizedScienceType(Base):
    __tablename__ = 'NormalizedScienceTypes'

    id = Column(Integer, primary_key=True)
    cycle_id = Column(Integer, nullable=False)
    science_type = Column(Text, nullable=False)
    normalized = Column(Integer, nullable=False)


class RESOURCE(Base):
    __tablename__ = 'RESOURCES'

    resource_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    RESOURCE_GROUP = Column(Text)
    RESOURCE_NAME = Column(Text)
    PROPOSAL_ID = Column(ForeignKey('proposal.proposal_id'), nullable=False, index=True)
    TELESCOPE = Column(String(255), nullable=False)
    DISPLAY_POSITION = Column(Float(asdecimal=True), nullable=False)

    proposal = relationship('Proposal')


class GMVARESOURCE(RESOURCE):
    __tablename__ = 'GMVA_RESOURCE'

    Id = Column(ForeignKey('RESOURCES.resource_id', ondelete='CASCADE'), primary_key=True, index=True)
    FRONT_END = Column(String(255))
    BACK_END = Column(String(255))
    VLBA = Column(BIT(1), nullable=False)
    EUROPEAN = Column(BIT(1), nullable=False)
    BREWSTER = Column(BIT(1), nullable=False)
    FORT_DAVIS = Column(BIT(1), nullable=False)
    KITT_PEAK = Column(BIT(1), nullable=False)
    LOS_ALAMOS = Column(BIT(1), nullable=False)
    MAUNA_KEA = Column(BIT(1), nullable=False)
    NORTH_LIBERTY = Column(BIT(1), nullable=False)
    OWENS_VALLEY = Column(BIT(1), nullable=False)
    PIE_TOWN = Column(BIT(1), nullable=False)
    EFFELSBERG = Column(BIT(1), nullable=False)
    PLATEAU_DE_BURE = Column(BIT(1), nullable=False)
    PICO_VELETA = Column(BIT(1), nullable=False)
    ONSALA = Column(BIT(1), nullable=False)
    METSAHOVI = Column(BIT(1), nullable=False)
    YEBES = Column(BIT(1), nullable=False)
    OTHER_STATIONS = Column(String(255))
    FULL_POLAR = Column(BIT(1), nullable=False)
    AVERAGING_TIME = Column(String(255))
    SPECTRAL_POINTS = Column(String(255))
    MULTI_CORR_PASS = Column(String(255))
    MARK4_CONVERSION = Column(BIT(1), nullable=False)
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    SAMPLING_LEVEL = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    BITS = Column(String(255), nullable=False)
    POLAR_CROSS_PROD = Column(String(255))
    AGG_BIT_RATE = Column(String(255))
    NO_OF_FIELDS = Column(Integer)
    Wideband_Observing_System = Column(BIT(1), nullable=False)
    Observing_System = Column(String(255), nullable=False)
    GREEN_BANK = Column(BIT(1), nullable=False)


class GBTRESOURCE(RESOURCE):
    __tablename__ = 'GBT_RESOURCE'

    Id = Column(ForeignKey('RESOURCES.resource_id', ondelete='CASCADE'), primary_key=True, index=True)
    FRONT_END = Column(String(255))
    BACK_END = Column(String(255))
    RECEIVER_OTHER = Column(String(255))


class GBTGuppi(GBTRESOURCE):
    __tablename__ = 'GBT_Guppi'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    COMMENTS = Column(Text)
    BAND_WIDTH = Column(String(128))
    BAND_WIDTH_UNIT = Column(String(128))
    OBSERVING_MODE = Column(String(128))
    POLAR_CROSS_PROD = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    Dedispersion = Column(String(255))
    Bits = Column(String(255), server_default=text("'1'"))


class GBTBACKENDBREAKTHROUGHLISTEN(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_BREAKTHROUGH_LISTEN'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    COMMENTS = Column(String(512))


class GBTBACKENDPROCESSORPULSAR(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_PROCESSOR_PULSAR'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    BITS = Column(String(255), nullable=False)
    SPECTRAL_RESOLUTION = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    POLAR_CROSS_PROD = Column(String(255))
    PHASE_BINS = Column(String(255))


class GBTZpectrometer(GBTRESOURCE):
    __tablename__ = 'GBT_Zpectrometer'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    COMMENTS = Column(Text)
    PI_PRE_APPROVAL = Column(BIT(1))


class GBTVEGAS14A(GBTRESOURCE):
    __tablename__ = 'GBT_VEGAS_14A'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    observing_type = Column(String(255))
    number_beams = Column(Integer)


class GBTBACKENDRADAR(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_RADAR'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))
    PI_PRE_APPROVAL = Column(BIT(1))
    SPECTRAL_RESOLUTION = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    Bits = Column(String(255), server_default=text("'1'"))


class GBTSPECTROSPIGOT(GBTRESOURCE):
    __tablename__ = 'GBT_SPECTRO_SPIGOT'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    POLAR_CROSS_PRODUCT = Column(String(255))
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))
    BITS = Column(String(255), nullable=False)
    SPECTRAL_RESOLUTION = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))


class GBTBACKENDCGSR2(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_CGSR2'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    PI_PRE_APPROVAL = Column(BIT(1))
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))


class GBTBACKENDPROCESSORLINE(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_PROCESSOR_LINE'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    BITS = Column(String(255), nullable=False)
    SPECTRAL_RESOLUTION = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    POLAR_CROSS_PROD = Column(String(255))


class GBTBACKENDGASP(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_GASP'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    PI_PRE_APPROVAL = Column(BIT(1))
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))


class GBTBACKENDBCPM(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_BCPM'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))


class GBTSPECTOMETERLINE(GBTRESOURCE):
    __tablename__ = 'GBT_SPECTOMETER_LINE'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    BITS = Column(String(255), nullable=False)
    SPECTRAL_RESOLUTION = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    SAMPLING_LEVEL = Column(String(255))
    BEAMS = Column(String(255))
    POLAR_CROSS_PRODUCT = Column(String(255))


class GBTVEGA(GBTRESOURCE):
    __tablename__ = 'GBT_VEGAS'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    BITS = Column(String(255), nullable=False)
    SPECTRAL_RESOLUTION = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    CROSS_POLARIZATION = Column(BIT(1))
    BEAMS = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    Resolution = Column(String(255))
    MinimumInterval = Column(String(255))


class GBTBACKENDJPLRADAR(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_JPL_RADAR'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))
    PI_PRE_APPROVAL = Column(BIT(1))
    SPECTRAL_RESOLUTION = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    BITS = Column(String(255))


class GBTBACKENDDIGITAL(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_DIGITAL'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    Bits = Column(String(255), server_default=text("'1'"))


class GBTMUSTANG(GBTRESOURCE):
    __tablename__ = 'GBT_MUSTANG'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    COMMENTS = Column(Text)
    Array = Column(String(255))


class GBTBACKENDOTHER(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_OTHER'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SPECIAL_BACKEND = Column(String(255))
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    BITS = Column(String(255), nullable=False)
    COMMENTS = Column(Text)


class GBTBACKENDCALTECHCONTINUUM(GBTRESOURCE):
    __tablename__ = 'GBT_BACKEND_CALTECH_CONTINUUM'

    Id = Column(ForeignKey('GBT_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String(255))
    SKY_FREQUENCY_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(String(255))
    TIME_RESOLUTION_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))


class VLBARESOURCE(RESOURCE):
    __tablename__ = 'VLBA_RESOURCE'

    Id = Column(ForeignKey('RESOURCES.resource_id', ondelete='CASCADE'), primary_key=True, index=True)
    FRONT_END = Column(String(255))
    BACK_END = Column(String(255))
    VLBA = Column(BIT(1), nullable=False)
    HSA = Column(BIT(1), nullable=False)
    BREWSTER = Column(BIT(1), nullable=False)
    FORT_DAVIS = Column(BIT(1), nullable=False)
    HANCOCK = Column(BIT(1), nullable=False)
    KITT_PEAK = Column(BIT(1), nullable=False)
    LOS_ALAMOS = Column(BIT(1), nullable=False)
    MAUNA_KEA = Column(BIT(1), nullable=False)
    NORTH_LIBERTY = Column(BIT(1), nullable=False)
    OWENS_VALLEY = Column(BIT(1), nullable=False)
    PIE_TOWN = Column(BIT(1), nullable=False)
    ST_CROIX = Column(BIT(1), nullable=False)
    GBT = Column(BIT(1), nullable=False, index=True)
    ARECIBO = Column(BIT(1), nullable=False)
    EFFELSBERG = Column(BIT(1), nullable=False)
    VLA_Y1 = Column(BIT(1), nullable=False, index=True)
    VLA_Y27 = Column(BIT(1), nullable=False, index=True)
    GEODETIC = Column(String(255))
    FULL_POLAR = Column(BIT(1), nullable=False)
    PULSAR_GATE = Column(BIT(1), nullable=False)
    MULTIPLE_FIELDS = Column(String(255))
    AVERAGING_TIME = Column(String(255))
    SPECTRAL_POINTS = Column(String(255))
    MULTI_CORR_PASS = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    SAMPLING_LEVEL = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    BITS = Column(String(255), nullable=False)
    POLAR_CROSS_PROD = Column(String(255))
    AGG_BIT_RATE = Column(String(255))
    NO_OF_FIELDS = Column(Integer, server_default=text("'0'"))
    Wideband_Observing_System = Column(BIT(1), nullable=False)
    Observing_Mode = Column(String(255), server_default=text("'Standard'"))
    RSROComments = Column(String)
    Observing_System = Column(String(255), server_default=text("'Legacy System'"))
    MARK4_CONVERSION = Column(BIT(1))
    shared_risk = Column(BIT(1), nullable=False)


class VLARESOURCE(RESOURCE):
    __tablename__ = 'VLA_RESOURCE'

    Id = Column(ForeignKey('RESOURCES.resource_id', ondelete='CASCADE'), primary_key=True, index=True)
    FRONT_END = Column(String(255))
    BACK_END = Column(String(255))
    CONFIGURATION = Column(String(255))


class VLABACKENDGENERAL13A(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_GENERAL13A'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    OBSERVING_CHOICE = Column(String(255), nullable=False)
    FILE_CONTENTS = Column(MEDIUMBLOB)
    REST_FREQUENCIES = Column(String)


class VLABACKENDSRO13A(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_SRO13A'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SRO_CATALOG_ID = Column(String(255), nullable=False)
    SRO_RESOURCE_NAME = Column(String(255), nullable=False)
    SRO_RESOURCE_ID = Column(String(255), nullable=False)


class VLABACKENDOSRODUAL(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_OSRODUAL'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String)
    REST_FREQUENCY_TYPE = Column(String(255))
    SKY_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    HANNING_SMOOTH = Column(String(255))
    SPECTRAL_RESOL = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    IF_MODE = Column(SmallInteger)
    Polarization = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    CHANNEL_WIDTH = Column(String(255))
    CHANNEL_WIDTH_UNIT = Column(String(255))


class VLABACKENDSINGLE(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_SINGLE'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    SKY_FREQUENCY = Column(Text)
    SKY_UNIT = Column(String(255))
    REST_FREQUENCY_TYPE = Column(String(255))


class VLABACKENDECSO(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_ECSO'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    COMMENTS = Column(Text)


class VLABACKENDOSRO2(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_OSRO2'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(Text)
    REST_FREQUENCY_TYPE = Column(String(255))
    SKY_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    HANNING_SMOOTH = Column(String(255))
    SPECTRAL_RESOL = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    IF_MODE = Column(SmallInteger)
    Polarization = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    CHANNEL_WIDTH = Column(String(255))
    CHANNEL_WIDTH_UNIT = Column(String(255))


class VLABACKENDOSROFULL(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_OSROFULL'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String)
    REST_FREQUENCY_TYPE = Column(String(255))
    SKY_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    HANNING_SMOOTH = Column(String(255))
    SPECTRAL_RESOL = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    IF_MODE = Column(SmallInteger)
    Polarization = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    CHANNEL_WIDTH = Column(String(255))
    CHANNEL_WIDTH_UNIT = Column(String(255))


class VLABACKENDHIGH(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_HIGH'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SPECTRAL_RESOL = Column(Text)
    SPECTRAL_RESOL_UNIT = Column(String(255))
    TIME_RESOLUTION = Column(Text)
    TIME_RESOLUTION_UNIT = Column(String(255))


class VLABACKENDOSROSINGLE(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_OSROSINGLE'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(String)
    REST_FREQUENCY_TYPE = Column(String(255))
    SKY_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    HANNING_SMOOTH = Column(String(255))
    SPECTRAL_RESOL = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    IF_MODE = Column(SmallInteger)
    Polarization = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    CHANNEL_WIDTH = Column(String(255))
    CHANNEL_WIDTH_UNIT = Column(String(255))


class VLABACKENDOSRO1(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_OSRO1'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(Text)
    REST_FREQUENCY_TYPE = Column(String(255))
    SKY_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    HANNING_SMOOTH = Column(String(255))
    SPECTRAL_RESOL = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    IF_MODE = Column(SmallInteger)
    Polarization = Column(String(255))
    CHANNEL_COUNT = Column(Integer)
    CHANNEL_WIDTH = Column(String(255))
    CHANNEL_WIDTH_UNIT = Column(String(255))


class VLABACKENDRSRO13A(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_RSRO13A'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    DESCRIPTION = Column(String(255), nullable=False)


class VLABACKENDMULTI(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_MULTI'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SKY_FREQUENCY = Column(Text)
    REST_FREQUENCY_TYPE = Column(String(255))
    SKY_UNIT = Column(String(255))
    BAND_WIDTH = Column(String(255))
    BAND_WIDTH_UNIT = Column(String(255))
    HANNING_SMOOTH = Column(String(255))
    SPECTRAL_RESOL = Column(String(255))
    SPECTRAL_RESOL_UNIT = Column(String(255))
    IF_MODE = Column(SmallInteger)
    CHANNEL_COUNT = Column(Integer)


class VLABACKENDRSRO(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_RSRO'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    COMMENTS = Column(Text)


class VLABACKENDSTAFF13A(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_STAFF13A'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    DESCRIPTION = Column(String(255), nullable=False)


class VLABACKENDOTHER(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_OTHER'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    SPECIAL_BACK_END = Column(String(255))


class VLABACKENDWIDEBAND(VLARESOURCE):
    __tablename__ = 'VLA_BACKEND_WIDEBAND'

    Id = Column(ForeignKey('VLA_RESOURCE.Id', ondelete='CASCADE'), primary_key=True, index=True)
    BASEBAND_CENTERS = Column(String(255), nullable=False)
    POLARIZATION_PRODUCTS = Column(String(255), nullable=False)
    DUMP_TIME = Column(String(255), nullable=False)
    DATA_RATE = Column(String(255), nullable=False)
    BASEBANDS = Column(String(255), nullable=False)
    SHARED_RISK = Column(BIT(1), nullable=False)
    DATA_RATE_JUSTIFICATION = Column(String(255))


class TechJustification(Base):
    __tablename__ = 'TechJustification'

    id = Column(Integer, primary_key=True)
    proposal_id = Column(Integer, nullable=False, unique=True)
    arrayconfig = Column(Text)
    daynight = Column(Text)
    receivers = Column(Text)
    correlator = Column(Text)
    sensitivity = Column(Text)
    integrationTime = Column(Text)
    dumpTime = Column(Text)
    imaging = Column(Text)
    rfi = Column(Text)
    stations = Column(Text)
    weather = Column(Text)
    dates = Column(Text)
    minlength = Column(Text)
    phaseref = Column(Text)
    polarization = Column(Text)
    hsa = Column(Text)
    resource = Column(Text)
    mapping = Column(Text)
    overhead = Column(Text)
    novel = Column(Text)
    pulsar = Column(Text)
    other = Column(Text)
    mosaic = Column(Text)
    bitrate = Column(Text)
    joint = Column(Text)
    total_correlator_output_data_size = Column(Text)


class TechJustificationFile(Base):
    __tablename__ = 'TechJustificationFile'

    id = Column(Integer, primary_key=True)
    techjust_id = Column(Integer, nullable=False)
    filename = Column(Text, nullable=False)
    filetype = Column(Text, nullable=False)
    filesize = Column(Integer, nullable=False)
    file = Column(MEDIUMBLOB)


class ValidConfiguration(Base):
    __tablename__ = 'ValidConfigurations'

    id = Column(Integer, primary_key=True)
    cycle_id = Column(Integer, nullable=False)
    configuration = Column(Text, nullable=False)


class Addres(Base):
    __tablename__ = 'address'

    address_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    defaultAddress = Column(BIT(1))
    addressDescription = Column(String(255))
    street1 = Column(String(255))
    street2 = Column(String(255))
    street3 = Column(String(255))
    street4 = Column(String(255))
    city = Column(String(255))
    postalCode = Column(String(255))
    region = Column(String(255))
    addressState_id = Column(ForeignKey('addressState.addressState_id'), index=True)
    addressCountry_id = Column(ForeignKey('addressCountry.addressCountry_id'), index=True)
    person_id = Column(ForeignKey('person.person_id'), index=True)
    organization_id = Column(ForeignKey('organization.organization_id'), index=True)
    updatedOn = Column(DateTime)
    updatedBy = Column(Integer)

    addressCountry = relationship('AddressCountry')
    addressState = relationship('AddressState')
    organization = relationship('Organization')
    person = relationship('Person')


class AddressCountry(Base):
    __tablename__ = 'addressCountry'

    addressCountry_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    addressCountry = Column(String(255), nullable=False)
    alpha_2 = Column(String(2))
    alpha_3 = Column(String(3))
    idd_isd = Column(String(25))
    status = Column(String(25))


t_addressCountry_bck = Table(
    'addressCountry_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class AddressState(Base):
    __tablename__ = 'addressState'

    addressState_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    stateName = Column(String(255), nullable=False)
    state = Column(String(255))


t_addressState_bck = Table(
    'addressState_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class AddressType(Base):
    __tablename__ = 'addressType'

    addressType_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    addressType = Column(String(255), nullable=False)

    addresss = relationship('Addres', secondary='address_addressType')


t_addressType_bck = Table(
    'addressType_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


t_address_addressType = Table(
    'address_addressType', metadata,
    Column('address_id', ForeignKey('address.address_id'), primary_key=True, nullable=False, index=True),
    Column('addressType_id', ForeignKey('addressType.addressType_id'), primary_key=True, nullable=False, index=True)
)


t_address_bck = Table(
    'address_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class Author(Base):
    __tablename__ = 'author'

    author_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    AFFILIATION = Column(String(255))
    DOMESTIC = Column(BIT(1))
    NEW_USER = Column(BIT(1))
    EMAIL = Column(String(255), nullable=False)
    ADDRESS = Column(String(255))
    FIRST_NAME = Column(String(255))
    LAST_NAME = Column(String(255))
    PROFESSIONAL_STATUS = Column(String(255))
    THESIS_OBSERVING = Column(BIT(1), nullable=False)
    GRADUATION_YEAR = Column(String(255))
    TELEPHONE = Column(String(255))
    OLDAUTHOR_ID = Column(String(255), nullable=False)
    DISPLAY_POSITION = Column(Integer, nullable=False)
    STORAGE_ORDER = Column(Integer, nullable=False)
    OTHER_AWARDS = Column(Text)
    SUPPORT_REQUESTER = Column(BIT(1), nullable=False)
    SUPPORTED = Column(BIT(1), nullable=False)
    BUDGET = Column(Float(asdecimal=True))
    ASSIGNMENT = Column(Text)
    proposal_id = Column(ForeignKey('proposal.proposal_id'), nullable=False, index=True)
    user_id = Column(ForeignKey('userAuthentication.userAuthentication_id'), index=True)
    dissertationPlan_id = Column(ForeignKey('dissertationPlan.dissertationPlan_id'), index=True)
    person_id = Column(BigInteger, nullable=False)
    country_id = Column(ForeignKey('addressCountry.addressCountry_id'), index=True)
    state_id = Column(ForeignKey('addressState.addressState_id'), index=True)
    organization_id = Column(ForeignKey('organization.organization_id'), index=True)

    country = relationship('AddressCountry')
    dissertationPlan = relationship('DissertationPlan')
    organization = relationship('Organization')
    proposal = relationship('Proposal', primaryjoin='Author.proposal_id == Proposal.proposal_id', lazy='subquery')
    state = relationship('AddressState')
    user = relationship('UserAuthentication', lazy='subquery')


t_author_bck = Table(
    'author_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class Bug(Base):
    __tablename__ = 'bug'

    bug_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    id = Column(Integer, nullable=False)
    status = Column(String(255), nullable=False)
    statusNumber = Column(Integer, nullable=False, index=True)
    type = Column(String(255), nullable=False)
    typeNumber = Column(Integer, nullable=False, index=True)
    issueDate = Column(Date, index=True)
    completeDate = Column(Date, index=True)
    author = Column(String(255))
    location = Column(String(255))
    description = Column(Text)
    resolution = Column(Text)
    versionNumber = Column(String(255))
    osName = Column(String(255))
    email = Column(String(255))


class CasaRegionSetup(Base):
    __tablename__ = 'casaRegionSetup'

    regionSetup_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    region = Column(String(128), nullable=False, index=True)
    userName = Column(String(128), nullable=False, index=True)
    firstName = Column(String(128), nullable=False)
    lastName = Column(String(128), nullable=False)
    email = Column(Text, nullable=False)
    user_id = Column(ForeignKey('userAuthentication.userAuthentication_id'), index=True)

    user = relationship('UserAuthentication')


class CasaUserSetup(Base):
    __tablename__ = 'casaUserSetup'

    userSetup_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    role_id = Column(ForeignKey('role.role_id'), nullable=False, index=True)
    user_id = Column(ForeignKey('userAuthentication.userAuthentication_id'), nullable=False, index=True)
    region_id = Column(ForeignKey('casaRegionSetup.regionSetup_id'), nullable=False, index=True)
    region = Column(String(128), nullable=False, index=True)
    userName = Column(String(128), nullable=False, index=True)
    firstName = Column(String(128), nullable=False)
    lastName = Column(String(128), nullable=False)
    email = Column(Text, nullable=False)

    region1 = relationship('CasaRegionSetup')
    role = relationship('Role')
    user = relationship('UserAuthentication')


class CasaBug(Base):
    __tablename__ = 'casa_bug'

    bug_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    id = Column(Integer, nullable=False)
    status = Column(String(255), nullable=False)
    statusNumber = Column(Integer, nullable=False, index=True)
    type = Column(String(255), nullable=False)
    typeNumber = Column(Integer, nullable=False, index=True)
    issueDate = Column(Date, index=True)
    completeDate = Column(Date, index=True)
    author = Column(String(255))
    location = Column(String(255))
    description = Column(Text)
    resolution = Column(Text)
    versionNumber = Column(String(255))
    osName = Column(String(255))
    email = Column(String(255))
    region_id = Column(ForeignKey('casaRegionSetup.regionSetup_id'), index=True)
    regionName = Column(String(255))
    uss_id = Column(ForeignKey('casaUserSetup.userSetup_id'), index=True)
    username = Column(String(255))

    region = relationship('CasaRegionSetup')
    uss = relationship('CasaUserSetup')


class Casalog(Base):
    __tablename__ = 'casalog'

    casalog_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    date = Column(DateTime, nullable=False, index=True)
    pagename = Column(String(50), nullable=False)
    module = Column(String(20), nullable=False)
    action = Column(String(24))
    ipaddress = Column(String(128))
    username = Column(String(255), nullable=False, index=True)


class DissertationPlan(Base):
    __tablename__ = 'dissertationPlan'

    dissertationPlan_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    FILE_NAME = Column(String(255), nullable=False)
    FILE_ID = Column(String(255), nullable=False)
    BYTE_SIZE = Column(Integer)
    PAGE_COUNT = Column(Integer)
    LAST_DATE = Column(DateTime)
    CONTENT_TYPE = Column(String(255), nullable=False)
    FILE_TYPE = Column(String(255))
    FILE_CONTENTS = Column(MEDIUMBLOB)
    PERSON_ID = Column(ForeignKey('person.person_id'), unique=True)
    USER_ID = Column(ForeignKey('userAuthentication.userAuthentication_id'), unique=True)

    person = relationship('Person')
    userAuthentication = relationship('UserAuthentication')


class Email(Base):
    __tablename__ = 'email'

    email_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    emailType = Column(String(255))
    email = Column(String(255), nullable=False)
    defaultEmail = Column(BIT(1), nullable=False)
    person_id = Column(ForeignKey('person.person_id'), index=True)
    organization_id = Column(ForeignKey('organization.organization_id'), index=True)

    organization = relationship('Organization')
    person = relationship('Person')


t_email_bck = Table(
    'email_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class EntryStatu(Base):
    __tablename__ = 'entryStatus'

    entryStatus_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    entryStatus = Column(String(255), nullable=False)


t_entryStatus_bck = Table(
    'entryStatus_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


t_fix_15A_codes = Table(
    'fix_15A_codes', metadata,
    Column('proposal_id', BigInteger, nullable=False, server_default=text("'0'")),
    Column('prop_id', String(255), nullable=False),
    Column('new_id', String(320), nullable=False, server_default=text("''")),
    Column('pi_name', String(511)),
    Column('email', String(255), nullable=False),
    Column('deadline_date', DateTime)
)


class JustificationFile(Base):
    __tablename__ = 'justificationFile'

    justificationFile_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    JUSTIFICATION_FILE_NAME = Column(String(255), nullable=False)
    JUSTIFICATION_FILE_ID = Column(String(255), nullable=False)
    JUSTIFICATION_BYTE_SIZE = Column(Integer)
    JUSTIFICATION_PAGE_COUNT = Column(Integer)
    JUST_STORAGE_DATE = Column(DateTime)
    CONTENT_TYPE = Column(String(255), nullable=False)
    JUSTIFICATION_FILE_TYPE = Column(String(255))
    FILE_CONTENTS = Column(MEDIUMBLOB)


t_justification_bck = Table(
    'justification_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class ObservingType(Base):
    __tablename__ = 'observingType'

    observingType_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    proposal_id = Column(ForeignKey('proposal.proposal_id'), index=True)
    display_position = Column(Integer)
    observingType = Column(String(255), nullable=False)

    proposal = relationship('Proposal')


t_observingType_bck = Table(
    'observingType_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class Organization(Base):
    __tablename__ = 'organization'

    organization_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    formalName = Column(String(255), nullable=False)
    shortName = Column(String(255))
    mailingName = Column(String(255))
    departmentName = Column(String(255))
    url = Column(String(255))
    contactPerson1 = Column(String(255))
    contactPerson2 = Column(String(255))
    parentId = Column(Integer)
    updatedOn = Column(DateTime)
    updatedBy = Column(Integer)
    entryStatus = Column(ForeignKey('entryStatus.entryStatus_id'), index=True)

    entryStatu = relationship('EntryStatu')
    persons = relationship('Person', secondary='person_organization')


class OrganizationType(Base):
    __tablename__ = 'organizationType'

    organizationType_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    organizationType = Column(String(255), nullable=False)

    organizations = relationship('Organization', secondary='organization_organizationType')


t_organizationType_bck = Table(
    'organizationType_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


t_organization_bck = Table(
    'organization_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


t_organization_organizationType = Table(
    'organization_organizationType', metadata,
    Column('organization_id', ForeignKey('organization.organization_id'), primary_key=True, nullable=False, index=True),
    Column('organizationType_id', ForeignKey('organizationType.organizationType_id'), primary_key=True, nullable=False, index=True)
)


class Pagelog(Base):
    __tablename__ = 'pagelog'

    pagelog_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    date = Column(DateTime, nullable=False, index=True)
    pagename = Column(String(50), nullable=False)
    module = Column(String(20), nullable=False)
    action = Column(String(255))
    recordable = Column(BIT(1))
    username = Column(String(255), nullable=False, index=True)
    sessionid = Column(Text)
    proposalid = Column(String(128))
    ipAddress = Column(String(255))


class Permissionset(Base):
    __tablename__ = 'permissionset'

    permissionset_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    home_view = Column(BIT(1), nullable=False)
    home_full = Column(BIT(1), nullable=False)
    proposalModule_view = Column(BIT(1), nullable=False)
    proposalModule_full = Column(BIT(1), nullable=False)
    myproposals_view = Column(BIT(1), nullable=False)
    myproposals_full = Column(BIT(1), nullable=False)
    proposalList_view = Column(BIT(1), nullable=False)
    proposalList_full = Column(BIT(1), nullable=False)
    settingsModule_view = Column(BIT(1), nullable=False)
    settingsModule_full = Column(BIT(1), nullable=False)
    myprofile_view = Column(BIT(1), nullable=False)
    myprofile_full = Column(BIT(1), nullable=False)
    roleList_view = Column(BIT(1), nullable=False)
    roleList_full = Column(BIT(1), nullable=False)
    profileList_view = Column(BIT(1), nullable=False)
    profileList_full = Column(BIT(1), nullable=False)
    organizationList_view = Column(BIT(1), nullable=False)
    organizationList_full = Column(BIT(1), nullable=False)
    bug_view = Column(BIT(1), nullable=False)
    bug_full = Column(BIT(1), nullable=False)
    analyticsModule_view = Column(BIT(1), nullable=False)
    analyticsModule_full = Column(BIT(1), nullable=False)
    a_proposal_view = Column(BIT(1), nullable=False)
    a_proposal_full = Column(BIT(1), nullable=False)
    a_casa_view = Column(BIT(1), nullable=False)
    a_casa_full = Column(BIT(1), nullable=False)
    casaModule_view = Column(BIT(1), nullable=False)
    casaModule_full = Column(BIT(1), nullable=False)
    casaHome_view = Column(BIT(1), nullable=False)
    casaHome_full = Column(BIT(1), nullable=False)
    casa_bugs_view = Column(BIT(1), nullable=False)
    casa_bugs_full = Column(BIT(1), nullable=False)
    casa_setup_view = Column(BIT(1), nullable=False)
    casa_setup_full = Column(BIT(1), nullable=False)
    pscAccess = Column(BIT(1), nullable=False)
    reviewCategory_view = Column(BIT(1), nullable=False)
    reviewCategory_full = Column(BIT(1), nullable=False)
    available_authors_view = Column(BIT(1), nullable=False)
    available_authors_full = Column(BIT(1), nullable=False)
    available_organizations_view = Column(BIT(1), nullable=False)
    available_organizations_full = Column(BIT(1), nullable=False)
    profilesModule_view = Column(BIT(1), nullable=False)
    profilesModule_full = Column(BIT(1), nullable=False)
    adminModule_view = Column(BIT(1), nullable=False)
    adminModule_full = Column(BIT(1), nullable=False)
    groupList_view = Column(BIT(1), nullable=False)
    groupList_full = Column(BIT(1), nullable=False)
    reviewsModule_view = Column(BIT(1), nullable=False)
    reviewsModule_full = Column(BIT(1), nullable=False)
    myreviews_view = Column(BIT(1), nullable=False)
    myreviews_full = Column(BIT(1), nullable=False)
    refereeSetup_view = Column(BIT(1), nullable=False)
    refereeSetup_full = Column(BIT(1), nullable=False)
    mergeAccounts = Column(BIT(1), nullable=False)
    editRefereeComments = Column(BIT(1), nullable=False)
    observationsModule_view = Column(BIT(1), nullable=False)
    observationsModule_full = Column(BIT(1), nullable=False)
    helpdeskModule_view = Column(BIT(1), nullable=False)
    helpdeskModule_full = Column(BIT(1), nullable=False)
    reviewCycles_view = Column(BIT(1), nullable=False)
    reviewCycles_full = Column(BIT(1), nullable=False)
    tech_review_panel_view = Column(BIT(1), nullable=False)
    tech_review_panel_full = Column(BIT(1), nullable=False)
    scientific_review_panel_view = Column(BIT(1), nullable=False)
    scientific_review_panel_full = Column(BIT(1), nullable=False)
    reviewsSetup_view = Column(BIT(1), nullable=False)
    reviewsSetup_full = Column(BIT(1), nullable=False)
    technical_review_categorySetup_view = Column(BIT(1), nullable=False)
    technical_review_categorySetup_full = Column(BIT(1), nullable=False)
    scientific_review_categorySetup_view = Column(BIT(1), nullable=False)
    scientific_review_categorySetup_full = Column(BIT(1), nullable=False)
    technical_refereeSetup_view = Column(BIT(1), nullable=False)
    technical_refereeSetup_full = Column(BIT(1), nullable=False)
    scientific_refereeSetup_view = Column(BIT(1), nullable=False)
    scientific_refereeSetup_full = Column(BIT(1), nullable=False)
    reviews_summary_view = Column(BIT(1), nullable=False)
    reviews_summary_full = Column(BIT(1), nullable=False)
    panel_reviews_view = Column(BIT(1), nullable=False)
    panel_reviews_full = Column(BIT(1), nullable=False)
    tac_reviews_view = Column(BIT(1), nullable=False)
    tac_reviews_full = Column(BIT(1), nullable=False)
    tac_members_view = Column(BIT(1), nullable=False)
    tac_members_full = Column(BIT(1), nullable=False)
    tac_panel_view = Column(BIT(1), nullable=False)
    tac_panel_full = Column(BIT(1), nullable=False)
    dataProcessingModule_view = Column(BIT(1), nullable=False)
    dataProcessingModule_full = Column(BIT(1), nullable=False)


t_permissionset_bck = Table(
    'permissionset_bck', metadata,
    Column('permissionset_id', BigInteger, nullable=False, server_default=text("'0'")),
    Column('objectversion', Integer, nullable=False, server_default=text("'0'")),
    Column('home_view', BIT(1), nullable=False),
    Column('home_full', BIT(1), nullable=False),
    Column('proposalModule_view', BIT(1), nullable=False),
    Column('proposalModule_full', BIT(1), nullable=False),
    Column('myproposals_view', BIT(1), nullable=False),
    Column('myproposals_full', BIT(1), nullable=False),
    Column('proposalList_view', BIT(1), nullable=False),
    Column('proposalList_full', BIT(1), nullable=False),
    Column('settingsModule_view', BIT(1), nullable=False),
    Column('settingsModule_full', BIT(1), nullable=False),
    Column('myprofile_view', BIT(1), nullable=False),
    Column('myprofile_full', BIT(1), nullable=False),
    Column('roleList_view', BIT(1), nullable=False),
    Column('roleList_full', BIT(1), nullable=False),
    Column('profileList_view', BIT(1), nullable=False),
    Column('profileList_full', BIT(1), nullable=False),
    Column('organizationList_view', BIT(1), nullable=False),
    Column('organizationList_full', BIT(1), nullable=False),
    Column('bug_view', BIT(1), nullable=False),
    Column('bug_full', BIT(1), nullable=False),
    Column('analyticsModule_view', BIT(1), nullable=False),
    Column('analyticsModule_full', BIT(1), nullable=False),
    Column('a_summary_view', BIT(1), nullable=False),
    Column('a_summary_full', BIT(1), nullable=False)
)


class Person(Base):
    __tablename__ = 'person'
    __table_args__ = (
        Index('person_i2', 'firstName', 'lastName'),
    )

    person_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    prefix = Column(ForeignKey('personPrefix.personPrefix_id'), index=True)
    firstName = Column(String(255))
    middleName = Column(String(255))
    lastName = Column(String(255))
    preferredName = Column(String(255))
    suffix = Column(ForeignKey('personSuffix.personSuffix_id'), index=True)
    aipsNumber = Column(SmallInteger)
    gender = Column(String(255))
    url = Column(String(255))
    unknownAffiliationName = Column(String(255))
    unknownAffiliationCountry = Column(ForeignKey('addressCountry.addressCountry_id'), index=True)
    personContactLevel_id = Column(ForeignKey('personContactLevel.personContactLevel_id'), index=True)
    entryStatus = Column(ForeignKey('entryStatus.entryStatus_id'), index=True)
    defaultOrganization_id = Column(ForeignKey('organization.organization_id'), index=True)
    personAuthentication_id = Column(ForeignKey('userAuthentication.userAuthentication_id'), index=True)
    personSpouse_id = Column(ForeignKey('personSpouse.personSpouse_id'), index=True)
    outOfDateReporterId = Column(Integer)
    updatedOn = Column(DateTime)
    updatedBy = Column(Integer)
    graduationYear = Column(SmallInteger)
    timezone = Column(String(255))
    lastLogin = Column(DateTime)
    lastObservation = Column(DateTime)
    incomplete = Column(BIT(1))
    affiliation = Column(Text)
    enabled = Column(BIT(1), nullable=False, index=True)
    gender_for_metrics = Column(String(255))

    defaultOrganization = relationship('Organization')
    entryStatu = relationship('EntryStatu')
    personAuthentication = relationship('UserAuthentication')
    personContactLevel = relationship('PersonContactLevel')
    personSpouse = relationship('PersonSpouse')
    personPrefix = relationship('PersonPrefix')
    personSuffix = relationship('PersonSuffix')
    addressCountry = relationship('AddressCountry')
    reviewCategorys = relationship('ReviewCategory', secondary='person_reviewCategory')


class PersonContactLevel(Base):
    __tablename__ = 'personContactLevel'

    personContactLevel_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    personContactLevel = Column(String(255), nullable=False)


t_personContactLevel_bck = Table(
    'personContactLevel_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class PersonGroup(Base):
    __tablename__ = 'personGroup'

    personGroup_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    personGroupType = Column(SmallInteger, nullable=False)
    personGroupName = Column(String(255), nullable=False)
    personGroupDescription = Column(String(255), nullable=False)

    persons = relationship('Person', secondary='person_personGroup')


t_personGroup_bck = Table(
    'personGroup_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class PersonNote(Base):
    __tablename__ = 'personNote'

    userNote_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    personNoteText = Column(String(255), nullable=False)
    updatedOn = Column(DateTime)
    updatedBy = Column(Integer)
    person_id = Column(ForeignKey('person.person_id'), index=True)

    person = relationship('Person')


t_personNote_bck = Table(
    'personNote_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class PersonPrefix(Base):
    __tablename__ = 'personPrefix'

    personPrefix_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    personPrefix = Column(String(255), nullable=False, unique=True)


t_personPrefix_bck = Table(
    'personPrefix_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class PersonSpouse(Base):
    __tablename__ = 'personSpouse'

    personSpouse_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    prefix = Column(ForeignKey('personPrefix.personPrefix_id'), index=True)
    firstName = Column(String(255))
    middleName = Column(String(255))
    lastName = Column(String(255))
    preferredName = Column(String(255))
    suffix = Column(ForeignKey('personSuffix.personSuffix_id'), index=True)
    gender = Column(String(1))

    personPrefix = relationship('PersonPrefix')
    personSuffix = relationship('PersonSuffix')


class PersonSuffix(Base):
    __tablename__ = 'personSuffix'

    personSuffix_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    personSuffix = Column(String(255), nullable=False, unique=True)


t_personSuffix_bck = Table(
    'personSuffix_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class PersonType(Base):
    __tablename__ = 'personType'

    personType_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    personType = Column(String(255), nullable=False)
    educationType = Column(BIT(1), nullable=False)

    persons = relationship('Person', secondary='person_personType')


t_personType_bck = Table(
    'personType_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


t_person_bck = Table(
    'person_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


t_person_organization = Table(
    'person_organization', metadata,
    Column('organization_id', ForeignKey('organization.organization_id'), primary_key=True, nullable=False, index=True),
    Column('person_id', ForeignKey('person.person_id'), primary_key=True, nullable=False, index=True)
)


t_person_personGroup = Table(
    'person_personGroup', metadata,
    Column('person_id', ForeignKey('person.person_id'), primary_key=True, nullable=False, index=True),
    Column('personGroup_id', ForeignKey('personGroup.personGroup_id', ondelete='CASCADE'), primary_key=True, nullable=False, index=True)
)


t_person_personType = Table(
    'person_personType', metadata,
    Column('person_id', ForeignKey('person.person_id'), primary_key=True, nullable=False, index=True),
    Column('personType_id', ForeignKey('personType.personType_id'), primary_key=True, nullable=False, index=True)
)


t_person_reviewCategory = Table(
    'person_reviewCategory', metadata,
    Column('person_id', ForeignKey('person.person_id'), primary_key=True, nullable=False, index=True),
    Column('reviewCategory_id', ForeignKey('reviewCategory.reviewCategory_id'), primary_key=True, nullable=False, index=True)
)


class Phone(Base):
    __tablename__ = 'phone'

    phone_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    phoneType = Column(String(255))
    phone = Column(String(255), nullable=False)
    defaultPhone = Column(BIT(1), nullable=False)
    fax = Column(BIT(1), nullable=False)
    person_id = Column(ForeignKey('person.person_id'), index=True)
    organization_id = Column(ForeignKey('organization.organization_id'), index=True)

    organization = relationship('Organization')
    person = relationship('Person')


t_phone_bck = Table(
    'phone_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class Proposal(Base):
    __tablename__ = 'proposal'

    proposal_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    ABSTRACT = Column(Text)
    CREATED_DATE = Column(DateTime, nullable=False)
    PROP_ID = Column(String(255), nullable=False, unique=True)
    LEGACY_ID = Column(String(255, 'latin1_general_cs'))
    PRESENT = Column(String(255))
    PROPOSAL_TYPE = Column(String(255))
    JOINT_PROPOSAL_TYPE = Column(String(255))
    RAPID_RESPONSE_TYPE = Column(String(255))
    STAFF_SUPPORT = Column(String(255))
    STATUS = Column(String(255))
    PLAN_SUBMITTED = Column(String(255))
    MODIFIED_DATE = Column(DateTime, nullable=False)
    SUBMITTED_DATE = Column(DateTime)
    DEADLINE_DATE = Column(DateTime)
    TITLE = Column(Text)
    PROCESSED = Column(BIT(1), nullable=False)
    RELATED_PROPOSALS = Column(Text)
    PREV_PROP_IDS = Column(Text)
    LOCK_USER_ID = Column(String(255))
    LOCK_MILLIS = Column(BigInteger)
    LOCK_USER_INFO = Column(String(255))
    OBSERVING_TYPE_OTHER = Column(String(255))
    OLD_CONTACT = Column(String(255))
    OLD_PI = Column(String(255))
    OLD_EDITOR = Column(String(255))
    justificationFile_id = Column(ForeignKey('justificationFile.justificationFile_id'), index=True)
    editor_id = Column(ForeignKey('author.author_id'), index=True)
    contact_id = Column(ForeignKey('author.author_id'), index=True)
    principal_investigator_id = Column(ForeignKey('author.author_id'), index=True)
    comments = Column(Text)
    category1_id = Column(ForeignKey('reviewCategory.reviewCategory_id'), index=True)
    category2_id = Column(ForeignKey('reviewCategory.reviewCategory_id'), index=True)
    TELESCOPE = Column(String(255), nullable=False)
    category3_id = Column(ForeignKey('reviewCategory.reviewCategory_id'), index=True)
    displayTechnicalReviews = Column(BIT(1))
    reviewersConflict = Column(BIT(1))
    technicalCategory_id = Column(ForeignKey('technical_review_category.tech_reviewCategory_id'), index=True)
    scienceCategory = Column(String(255))
    public = Column(BIT(1), nullable=False)
    allocated_hours = Column(Float(asdecimal=True), server_default=text("'0'"))
    disposition_letter = Column(MEDIUMBLOB)
    trigger_criteria = Column(String)
    techJustification_id = Column(Integer)
    uses_GBT = Column(BIT(1), nullable=False)
    uses_VLA = Column(BIT(1), nullable=False)
    allocated_hours_a = Column(Float(asdecimal=True), server_default=text("'0'"))
    allocated_hours_b = Column(Float(asdecimal=True), server_default=text("'0'"))
    allocated_hours_c = Column(Float(asdecimal=True), server_default=text("'0'"))
    sponsor = Column(String(255))
    hst_orbits_requested = Column(String(255))
    external = Column(BIT(1), nullable=False)
    total_time = Column(Float(asdecimal=True), server_default=text("'0'"))
    external_proposal_id = Column(String(255))
    swift_ksecs = Column(String(255))
    chandra_ksecs = Column(String(255))

    category1 = relationship('ReviewCategory', primaryjoin='Proposal.category1_id == ReviewCategory.reviewCategory_id')
    category2 = relationship('ReviewCategory', primaryjoin='Proposal.category2_id == ReviewCategory.reviewCategory_id')
    category3 = relationship('ReviewCategory', primaryjoin='Proposal.category3_id == ReviewCategory.reviewCategory_id')
    contact = relationship('Author', primaryjoin='Proposal.contact_id == Author.author_id')
    editor = relationship('Author', primaryjoin='Proposal.editor_id == Author.author_id')
    justificationFile = relationship('JustificationFile')
    principal_investigator = relationship('Author', primaryjoin='Proposal.principal_investigator_id == Author.author_id')
    technicalCategory = relationship('TechnicalReviewCategory')


t_proposal_15A_bck = Table(
    'proposal_15A_bck', metadata,
    Column('proposal_id', BigInteger, nullable=False, server_default=text("'0'")),
    Column('objectversion', Integer, nullable=False),
    Column('ABSTRACT', Text),
    Column('CREATED_DATE', DateTime, nullable=False),
    Column('PROP_ID', String(255), nullable=False),
    Column('LEGACY_ID', String(255, 'latin1_general_cs')),
    Column('PRESENT', String(255)),
    Column('PROPOSAL_TYPE', String(255)),
    Column('JOINT_PROPOSAL_TYPE', String(255)),
    Column('RAPID_RESPONSE_TYPE', String(255)),
    Column('STAFF_SUPPORT', String(255)),
    Column('STATUS', String(255)),
    Column('PLAN_SUBMITTED', String(255)),
    Column('MODIFIED_DATE', DateTime, nullable=False),
    Column('SUBMITTED_DATE', DateTime),
    Column('DEADLINE_DATE', DateTime),
    Column('TITLE', Text),
    Column('PROCESSED', BIT(1), nullable=False),
    Column('RELATED_PROPOSALS', Text),
    Column('PREV_PROP_IDS', Text),
    Column('LOCK_USER_ID', String(255)),
    Column('LOCK_MILLIS', BigInteger),
    Column('LOCK_USER_INFO', String(255)),
    Column('OBSERVING_TYPE_OTHER', String(255)),
    Column('OLD_CONTACT', String(255)),
    Column('OLD_PI', String(255)),
    Column('OLD_EDITOR', String(255)),
    Column('justificationFile_id', BigInteger),
    Column('editor_id', BigInteger),
    Column('contact_id', BigInteger),
    Column('principal_investigator_id', BigInteger),
    Column('comments', Text),
    Column('category1_id', BigInteger),
    Column('category2_id', BigInteger),
    Column('TELESCOPE', String(255), nullable=False),
    Column('category3_id', BigInteger),
    Column('displayTechnicalReviews', BIT(1)),
    Column('reviewersConflict', BIT(1)),
    Column('technicalCategory_id', BigInteger),
    Column('scienceCategory', String(255)),
    Column('public', BIT(1), nullable=False),
    Column('allocated_hours', Float(asdecimal=True), server_default=text("'0'")),
    Column('disposition_letter', MEDIUMBLOB),
    Column('trigger_criteria', String),
    Column('techJustification_id', Integer),
    Column('uses_GBT', BIT(1), nullable=False),
    Column('uses_VLA', BIT(1), nullable=False),
    Column('allocated_hours_a', Float(asdecimal=True), server_default=text("'0'")),
    Column('allocated_hours_b', Float(asdecimal=True), server_default=text("'0'")),
    Column('allocated_hours_c', Float(asdecimal=True), server_default=text("'0'")),
    Column('sponsor', String(255)),
    Column('hst_orbits_requested', String(255))
)


t_proposal_bck = Table(
    'proposal_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class ProposalNormalizedScore(Base):
    __tablename__ = 'proposal_normalized_scores'

    normalized_scores_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    review_cycle_id = Column(ForeignKey('review_cycles.review_cycle_id'), nullable=False, index=True)
    proposal_id = Column(ForeignKey('proposal.proposal_id'), nullable=False, index=True)
    scientificCategory = Column(String(255), nullable=False)
    avgNormalizedScore = Column(Float(asdecimal=True), nullable=False)
    normalizedScoreStdev = Column(Float(asdecimal=True), nullable=False)
    srpScores = Column(Float(asdecimal=True), nullable=False)
    complete = Column(BIT(1), nullable=False)
    finalized = Column(BIT(1), nullable=False)

    proposal = relationship('Proposal')
    review_cycle = relationship('ReviewCycle')


t_proposal_normalized_scores_backup = Table(
    'proposal_normalized_scores_backup', metadata,
    Column('normalized_scores_id', BigInteger, nullable=False, server_default=text("'0'")),
    Column('objectversion', Integer, nullable=False),
    Column('review_cycle_id', BigInteger, nullable=False),
    Column('proposal_id', BigInteger, nullable=False),
    Column('scientificCategory', String(255), nullable=False),
    Column('avgNormalizedScore', Float(asdecimal=True), nullable=False),
    Column('normalizedScoreStdev', Float(asdecimal=True), nullable=False),
    Column('srpScores', Float(asdecimal=True), nullable=False),
    Column('complete', BIT(1), nullable=False),
    Column('finalized', BIT(1), nullable=False)
)


t_proposal_public_bit_allocated_hours_backup = Table(
    'proposal_public_bit_allocated_hours_backup', metadata,
    Column('proposal_id', BigInteger, nullable=False, server_default=text("'0'")),
    Column('public', BIT(1), nullable=False),
    Column('allocated_hours', Float(asdecimal=True), server_default=text("'0'"))
)


class ProposalReview(Base):
    __tablename__ = 'proposal_reviews'

    review_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    complete = Column(BIT(1), nullable=False)
    chair = Column(BIT(1), nullable=False)
    conflict = Column(BIT(1), nullable=False)
    reasons = Column(String)
    review_cycle_id = Column(ForeignKey('review_cycles.review_cycle_id'), nullable=False, index=True)
    referee_id = Column(ForeignKey('referees.referee_id'), nullable=False, index=True)
    proposal_id = Column(ForeignKey('proposal.proposal_id'), nullable=False, index=True)
    saved = Column(BIT(1), nullable=False)
    close = Column(BIT(1), nullable=False)
    reviewsComplete = Column(BIT(1), nullable=False)

    proposal = relationship('Proposal')
    referee = relationship('Referee')
    review_cycle = relationship('ReviewCycle')


class ScientificReview(ProposalReview):
    __tablename__ = 'scientific_reviews'

    review_id = Column(ForeignKey('proposal_reviews.review_id', ondelete='CASCADE'), primary_key=True, index=True)
    scientificCategory = Column(String(255), nullable=False)
    pctRecommendTime = Column(Float(asdecimal=True))
    score = Column(Float(asdecimal=True), server_default=text("'0'"))
    comments = Column(String)
    normalizedScore = Column(Float(asdecimal=True), nullable=False, server_default=text("'0'"))


class TechnicalReview(ProposalReview):
    __tablename__ = 'technical_reviews'

    review_id = Column(ForeignKey('proposal_reviews.review_id', ondelete='CASCADE'), primary_key=True, index=True)
    commentsForAuthors = Column(String)
    commentsForTAC = Column(String)


class ProposalTacReview(Base):
    __tablename__ = 'proposal_tac_reviews'

    tac_review_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    review_cycle_id = Column(ForeignKey('review_cycles.review_cycle_id'), nullable=False, index=True)
    proposal_id = Column(ForeignKey('proposal.proposal_id'), nullable=False, index=True)
    totalTimeRequested = Column(Float(asdecimal=True), nullable=False)
    totalTimeAllocated = Column(Float(asdecimal=True), nullable=False)
    schedulingPriority = Column(String(255), nullable=False)
    complete = Column(BIT(1), nullable=False)
    finalized = Column(BIT(1), nullable=False)
    normalizedSRPScore = Column(Float(asdecimal=True), nullable=False, server_default=text("'0'"))
    reviewPostponed = Column(BIT(1), nullable=False)

    proposal = relationship('Proposal')
    review_cycle = relationship('ReviewCycle')


# class RefereeComment(Base):
#     __tablename__ = 'refereeComments'
#
#     refereeComments_id = Column(BigInteger, primary_key=True)
#     objectversion = Column(Integer, nullable=False)
#     person_id = Column(ForeignKey('person.person_id'), index=True)
#     comments = Column(Text)
#     pctRecommendTime = Column(String(255))
#     rating = Column(String(255))
#     commentsToPSC = Column(Text)
#     complete = Column(BIT(1), nullable=False)
#
#     person = relationship('Person')
#
#
# class RefereeComment(Base):
#     __tablename__ = 'referee_comment'
#
#     proposal_id = Column(ForeignKey('proposal.proposal_id'), primary_key=True, nullable=False, index=True)
#     commentid = Column(ForeignKey('refereeComments.refereeComments_id'), nullable=False, index=True)
#     userid = Column(ForeignKey('userAuthentication.userAuthentication_id'), primary_key=True, nullable=False, index=True)
#
#     parent = relationship('RefereeComment', remote_side=[proposal_id, userid])
#     proposal = relationship('Proposal')
#     userAuthentication = relationship('UserAuthentication')


class Referee(Base):
    __tablename__ = 'referees'

    referee_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    person_id = Column(ForeignKey('person.person_id'), nullable=False, index=True)
    user_id = Column(ForeignKey('userAuthentication.userAuthentication_id'), nullable=False, index=True)
    active = Column(BIT(1), nullable=False)
    acceptReviews = Column(BIT(1), nullable=False)
    type = Column(Integer, nullable=False)

    person = relationship('Person')
    user = relationship('UserAuthentication')


t_resource_bck = Table(
    'resource_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False),
    Column('class', String(255), nullable=False)
)


class ReviewCategory(Base):
    __tablename__ = 'reviewCategory'

    reviewCategory_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    telescope = Column(String(255), nullable=False)
    code = Column(String(128), nullable=False, index=True)
    description = Column(Text)


class ReviewCycle(Base):
    __tablename__ = 'review_cycles'

    review_cycle_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    title = Column(String(255), nullable=False)
    proposal_deadline = Column(Date, nullable=False)
    review_deadline = Column(Date, nullable=False)
    status = Column(String(255), nullable=False)
    code = Column(String(3), nullable=False, server_default=text("'xxx'"))


class Role(Base):
    __tablename__ = 'role'

    role_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    name = Column(String(128), nullable=False, index=True)
    description = Column(Text)
    permissionset_id = Column(ForeignKey('permissionset.permissionset_id'), nullable=False, index=True)

    permissionset = relationship('Permissionset')


class ScientificCategory(Base):
    __tablename__ = 'scientificCategory'

    scientificCategory_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    proposal_id = Column(ForeignKey('proposal.proposal_id'), index=True)
    display_position = Column(Integer)
    scientificCategory = Column(String(255), nullable=False)

    proposal = relationship('Proposal')


class ScientificReviewPanel(Base):
    __tablename__ = 'scientific_review_panel'

    sr_panel_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    review_cycle_id = Column(ForeignKey('review_cycles.review_cycle_id'), nullable=False, index=True)
    referee_id = Column(ForeignKey('referees.referee_id'), nullable=False, index=True)
    chair = Column(BIT(1), nullable=False)
    scientificCategory = Column(String(255), nullable=False)

    referee = relationship('Referee')
    review_cycle = relationship('ReviewCycle')


t_scientific_reviews_backup = Table(
    'scientific_reviews_backup', metadata,
    Column('review_id', BigInteger, nullable=False),
    Column('scientificCategory', String(255), nullable=False),
    Column('pctRecommendTime', Float(asdecimal=True)),
    Column('score', Float(asdecimal=True), server_default=text("'0'")),
    Column('comments', String),
    Column('normalizedScore', Float(asdecimal=True), nullable=False, server_default=text("'0'"))
)


t_scientific_type = Table(
    'scientific_type', metadata,
    Column('code', String(16), index=True),
    Column('name', String(50), index=True),
    Column('abbreviation', String(3)),
    Column('description', String(255)),
    Column('legacy', Integer)
)


class SearchPreference(Base):
    __tablename__ = 'searchPreferences'

    preference_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    username = Column(String(128), nullable=False)
    pagename = Column(Text, nullable=False)
    listname = Column(String(255))
    value = Column(String(255))


class Session(Base):
    __tablename__ = 'session'

    session_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    SESSION_NAME = Column(Text)
    SESSION_TIME = Column(Float(asdecimal=True))
    REPEATS = Column(Integer)
    SEPARATION = Column(String(255))
    INTERVAL_TIME = Column(Integer)
    CONSTRAINT_FIELD = Column(Text)
    COMMENTS = Column(Text)
    MINIMUM_LST = Column(Text)
    MAXIMUM_LST = Column(Text)
    ELEVATION_MINIMUM = Column(Text)
    SESSION_TIME_CALCULATED = Column(BIT(1), nullable=False)
    DISPLAY_POSITION = Column(Integer, nullable=False)
    PROPOSAL_ID = Column(ForeignKey('proposal.proposal_id'), nullable=False, index=True)
    MODIFIED_SESSION = Column(BIT(1), nullable=False)

    proposal = relationship('Proposal')


class ModifiedSession(Session):
    __tablename__ = 'ModifiedSession'

    session_id = Column(ForeignKey('session.session_id'), primary_key=True, index=True)
    CREATED_DATE = Column(DateTime, nullable=False)
    REQUESTED_SESSION_TIME = Column(Float(asdecimal=True))
    REQUESTED_REPEATS = Column(Integer)
    schedulingPriority = Column(String(255), server_default=text("''"))
    ALLOCATED_SESSION_TIME = Column(Float(asdecimal=True), server_default=text("'0'"))
    ALLOCATED_REPEATS = Column(Integer, server_default=text("'0'"))
    START_LST = Column(String)
    STOP_LST = Column(String)


class SessionPair(Base):
    __tablename__ = 'sessionPair'

    sessionPair_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    PAIR_IDENTIFIER = Column(String(255))
    FIGURE_OF_MERIT = Column(Text)
    FIGURE_UNITS = Column(String(255))
    PAIR_TIME = Column(Float(asdecimal=True))
    PAIR_TIME_UNIT = Column(String(255))
    CONSTRAINT_FIELD = Column(Text)
    DISPLAY_POSITION = Column(Integer, nullable=False)
    RESOURCE_GROUP = Column(String(255))
    SOURCE_GROUP = Column(String(255))
    SESSION_ID = Column(ForeignKey('session.session_id', ondelete='CASCADE'), nullable=False, index=True)
    SUB_ARRAY = Column(Integer, nullable=False)
    SUB_ARRAY_COUNT = Column(Integer, nullable=False)
    SUB_ARRAY_GROUP = Column(String(255), nullable=False)

    session = relationship('Session')


t_sessionPair_bck = Table(
    'sessionPair_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class SessionResource(Base):
    __tablename__ = 'sessionResource'

    sessionResource_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    sessionPair_id = Column(ForeignKey('sessionPair.sessionPair_id', ondelete='CASCADE'), nullable=False, index=True)
    resource_id = Column(ForeignKey('RESOURCES.resource_id', ondelete='CASCADE'), nullable=False, index=True)
    display_position = Column(Integer, nullable=False)
    SESSION_ID = Column(ForeignKey('session.session_id', ondelete='CASCADE'), nullable=False, index=True)

    session = relationship('Session')
    resource = relationship('RESOURCE')
    sessionPair = relationship('SessionPair')


t_sessionResource_bck = Table(
    'sessionResource_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class SessionSource(Base):
    __tablename__ = 'sessionSource'

    sessionSource_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    sessionPair_id = Column(ForeignKey('sessionPair.sessionPair_id', ondelete='CASCADE'), nullable=False, index=True)
    source_id = Column(ForeignKey('source.source_id', ondelete='CASCADE'), nullable=False, index=True)
    display_position = Column(Integer, nullable=False)
    SESSION_ID = Column(ForeignKey('session.session_id', ondelete='CASCADE'), nullable=False, index=True)

    session = relationship('Session')
    sessionPair = relationship('SessionPair')
    source = relationship('Source')


t_sessionSource_bck = Table(
    'sessionSource_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


t_session_bck = Table(
    'session_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class Source(Base):
    __tablename__ = 'source'

    source_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    proposal_id = Column(ForeignKey('proposal.proposal_id'), nullable=False, index=True)
    display_position = Column(Float(asdecimal=True), nullable=False, index=True)
    source_group = Column(Text)
    target_name = Column(Text)
    coordinate = Column(String(255))
    right_ascension = Column(Text)
    declination = Column(Text)
    right_ascension_range = Column(Text)
    declination_range = Column(Text)
    velocity_type = Column(String(255), nullable=False)
    velocity_redshift = Column(Text, nullable=False)
    convention = Column(String(255), nullable=False, server_default=text("'Radio'"))
    coordinate_system = Column(String(255), nullable=False, server_default=text("'Equatorial'"))
    referenceFrame = Column(String(255), nullable=False, server_default=text("'LSRK'"))
    calibrator = Column(BIT(1), nullable=False)

    proposal = relationship('Proposal')


t_source_bck = Table(
    'source_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class SrpComment(Base):
    __tablename__ = 'srp_comments'

    srp_comments_id = Column(BigInteger, primary_key=True)
    person_id = Column(ForeignKey('person.person_id'), nullable=False, index=True)
    referee_id = Column(ForeignKey('referees.referee_id'), index=True)
    commentsForAuthors = Column(String)
    commentsForTAC = Column(String)
    byPSTAdmin = Column(BIT(1), nullable=False)
    lastUpdated = Column(DateTime, nullable=False, server_default=text("CURRENT_TIMESTAMP"))

    person = relationship('Person')
    referee = relationship('Referee')


class SrpCommentsMap(Base):
    __tablename__ = 'srp_comments_map'

    normalized_scores_id = Column(ForeignKey('proposal_normalized_scores.normalized_scores_id'), primary_key=True, nullable=False, index=True)
    srp_comments_id = Column(ForeignKey('srp_comments.srp_comments_id'), nullable=False, index=True)
    person_id = Column(ForeignKey('person.person_id'), primary_key=True, nullable=False, index=True)

    normalized_scores = relationship('ProposalNormalizedScore')
    person = relationship('Person')
    srp_comments = relationship('SrpComment')


t_srp_comments_tmp = Table(
    'srp_comments_tmp', metadata,
    Column('srp_comments_id', BigInteger, nullable=False, server_default=text("'0'")),
    Column('person_id', BigInteger, nullable=False),
    Column('referee_id', BigInteger),
    Column('commentsForAuthors', String),
    Column('commentsForTAC', String),
    Column('byPSTAdmin', BIT(1), nullable=False),
    Column('lastUpdated', DateTime)
)


class StudentSupport(Base):
    __tablename__ = 'studentSupport'

    studentSupport_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    travel = Column(Text)
    hardware = Column(Text)
    miscellaneous_budget = Column(Float(asdecimal=True))
    applied = Column(BIT(1), nullable=False)
    propId = Column(String(255), nullable=False)
    proposal_id = Column(ForeignKey('proposal.proposal_id', ondelete='CASCADE'), unique=True)
    sos_pi_id = Column(ForeignKey('author.author_id'), index=True)

    proposal = relationship('Proposal')
    sos_pi = relationship('Author')


t_studentSupport_bck = Table(
    'studentSupport_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class TacComment(Base):
    __tablename__ = 'tac_comments'

    tac_comments_id = Column(BigInteger, primary_key=True)
    person_id = Column(ForeignKey('person.person_id'), nullable=False, index=True)
    referee_id = Column(ForeignKey('referees.referee_id'), index=True)
    commentsForAuthors = Column(String)
    commentsForTAC = Column(String)
    byPSTAdmin = Column(BIT(1), nullable=False)

    person = relationship('Person')
    referee = relationship('Referee')


class TacCommentsMap(Base):
    __tablename__ = 'tac_comments_map'

    tac_review_id = Column(ForeignKey('proposal_tac_reviews.tac_review_id'), primary_key=True, nullable=False, index=True)
    tac_comments_id = Column(ForeignKey('tac_comments.tac_comments_id'), nullable=False, index=True)
    person_id = Column(ForeignKey('person.person_id'), primary_key=True, nullable=False, index=True)

    person = relationship('Person')
    tac_comments = relationship('TacComment')
    tac_review = relationship('ProposalTacReview')


class TacPanel(Base):
    __tablename__ = 'tac_panel'

    tac_panel_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    review_cycle_id = Column(ForeignKey('review_cycles.review_cycle_id'), nullable=False, index=True)
    referee_id = Column(ForeignKey('referees.referee_id'), nullable=False, index=True)
    chair = Column(BIT(1), nullable=False)

    referee = relationship('Referee')
    review_cycle = relationship('ReviewCycle')


class TechnicalReviewCategory(Base):
    __tablename__ = 'technical_review_category'

    tech_reviewCategory_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    name = Column(String, nullable=False)
    code = Column(String(128), nullable=False, index=True)
    description = Column(String)
    telescope = Column(String(255), nullable=False)


class TechnicalReviewPanel(Base):
    __tablename__ = 'technical_review_panel'

    tr_panel_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    review_cycle_id = Column(ForeignKey('review_cycles.review_cycle_id'), nullable=False, index=True)
    referee_id = Column(ForeignKey('referees.referee_id'), nullable=False, index=True)
    technical_category_id = Column(ForeignKey('technical_review_category.tech_reviewCategory_id'), nullable=False, index=True)
    telescope = Column(String(255), nullable=False)

    referee = relationship('Referee')
    review_cycle = relationship('ReviewCycle')
    technical_category = relationship('TechnicalReviewCategory')


class Uisetting(Base):
    __tablename__ = 'uisetting'

    uisetting_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    FLYOVER_HELP = Column(BIT(1), nullable=False)
    TELESCOPE = Column(String(255))
    EXPIRATION_RECOVERY = Column(String(255))
    EDITOR_ROLE = Column(String(255))
    PROPOSALS_PER_PAGE = Column(Integer)
    CLIENT_SIDE_CHANGE_INDICATOR = Column(String(255))


t_uisettings_bck = Table(
    'uisettings_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


t_url_bck = Table(
    'url_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)


class UserAuthentication(Base):
    __tablename__ = 'userAuthentication'

    userAuthentication_id = Column(BigInteger, primary_key=True)
    objectversion = Column(Integer, nullable=False)
    role_id = Column(ForeignKey('role.role_id'), nullable=False, index=True)
    permissionset_id = Column(ForeignKey('permissionset.permissionset_id'), nullable=False, index=True)
    uisetting_id = Column(ForeignKey('uisetting.uisetting_id'), index=True)
    personName = Column(String(128), nullable=False, unique=True)
    password = Column(String(255), nullable=False)
    displayName = Column(String(255), nullable=False)
    authenticationToken = Column(String(255))
    changePassword = Column(BIT(1), nullable=False)
    rememberLogin = Column(BIT(1))
    defaultAccount_id = Column(ForeignKey('userAuthentication.userAuthentication_id'), index=True)

    defaultAccount = relationship('UserAuthentication', remote_side=[userAuthentication_id])
    permissionset = relationship('Permissionset')
    role = relationship('Role')
    uisetting = relationship('Uisetting')


t_userAuthentication_bck = Table(
    'userAuthentication_bck', metadata,
    Column('newid', BigInteger, nullable=False),
    Column('oldid', BigInteger, nullable=False)
)
